@@ -55,14 +55,14 @@
      tROOT4 = 271,
      tROOT3 = 272,
      tROOT = 273,
-     tFUNCTION = 274,
-     tVARIABLE = 275,
+     tVARIABLE = 274,
+     tNSUPNUM = 275,
      tSUPNUM = 276,
      tSUBNUM = 277,
      BOOLEAN_OPERATOR = 278,
      PERCENTAGE = 279,
      UNARY_MINUS = 280,
-     tINVERSE = 281
+     tIN = 281
    };
 #endif
 /* Tokens.  */
@@ -82,14 +82,14 @@
 #define tROOT4 271
 #define tROOT3 272
 #define tROOT 273
-#define tFUNCTION 274
-#define tVARIABLE 275
+#define tVARIABLE 274
+#define tNSUPNUM 275
 #define tSUPNUM 276
 #define tSUBNUM 277
 #define BOOLEAN_OPERATOR 278
 #define PERCENTAGE 279
 #define UNARY_MINUS 280
-#define tINVERSE 281
+#define tIN 281
 
 
 
@@ -99,7 +99,7 @@ typedef union YYSTYPE
 {
 
 /* Line 1676 of yacc.c  */
-#line 101 "./mp-equation-parser.y"
+#line 96 "./mp-equation-parser.y"
 
   MPNumber int_t;
   int integer;
